"use strict";
export * from "./rules";
export * from "./errors";
export * from "./types";
function evaluate(rules, value, obj) {
  let error;
  for (let i = 0; i < rules.length; i++) {
    const res = rules[i](value, obj);
    if (res) {
      error = res;
      break;
    }
  }
  return error;
}
function exists(value) {
  return value !== void 0 && value !== null;
}
function get(obj, path) {
  const tokens = path.split(".");
  return tokens.reduce(
    (o, k) => o && typeof o === "object" && exists(o[k]) ? o[k] : void 0,
    obj
  );
}
export function validate(obj, schema) {
  const errors = {};
  Object.entries(schema).forEach(([key, rules]) => {
    const err = evaluate(rules, get(obj, key), obj);
    if (!err)
      return;
    errors[key] = err;
  });
  if (Object.keys(errors).length > 0)
    return errors;
  return void 0;
}
